=== modified file 'drizzled/Makefile.am'
--- drizzled/Makefile.am	2009-05-08 19:07:39 +0000
+++ drizzled/Makefile.am	2009-05-08 19:27:21 +0000
@@ -585,7 +585,6 @@
 		show.cc \
 		sj_tmp_table.cc \
 		sql_base.cc \
-		sql_bitmap.cc \
 		sql_connect.cc \
 		sql_delete.cc \
 		sql_derived.cc \

=== modified file 'drizzled/drizzled.cc'
--- drizzled/drizzled.cc	2009-05-08 19:13:49 +0000
+++ drizzled/drizzled.cc	2009-05-08 19:27:21 +0000
@@ -367,7 +367,7 @@
 struct system_variables max_system_variables;
 struct system_status_var global_status_var;
 
-ThreadSafeBitset temp_pool;
+MY_BITMAP temp_pool;
 
 const CHARSET_INFO *system_charset_info, *files_charset_info ;
 const CHARSET_INFO *table_alias_charset;
@@ -591,6 +591,7 @@
   free(drizzle_tmpdir);
   if (opt_secure_file_priv)
     free(opt_secure_file_priv);
+  bitmap_free(&temp_pool);
 
   (void) unlink(pidfile_name);	// This may not always exist
 
@@ -1496,6 +1497,9 @@
   }
   global_system_variables.lc_time_names= my_default_lc_time_names;
 
+  if (use_temp_pool && bitmap_init(&temp_pool,0,1024,1))
+    return 1;
+
   /* Reset table_alias_charset, now that lower_case_table_names is set. */
   lower_case_table_names= 1; /* This we need to look at */
   table_alias_charset= files_charset_info;

=== modified file 'drizzled/field/blob.cc'
--- drizzled/field/blob.cc	2009-04-28 21:46:57 +0000
+++ drizzled/field/blob.cc	2009-05-08 19:27:21 +0000
@@ -605,7 +605,7 @@
   uint32_t const master_packlength=
     param_data > 0 ? param_data & 0xFF : packlength;
   uint32_t const length= get_length(from, master_packlength, low_byte_first);
-  table->write_set->set(field_index);
+  bitmap_set_bit(table->write_set, field_index);
   store(reinterpret_cast<const char*>(from) + master_packlength,
         length, field_charset);
   return(from + master_packlength + length);
@@ -677,11 +677,11 @@
 
 bool Field_blob::in_read_set()
 {
-  return (table->read_set->test(field_index));
+  return bitmap_is_set(table->read_set, field_index);
 }
 
 
 bool Field_blob::in_write_set()
 {
-  return (table->write_set->test(field_index));
+  return bitmap_is_set(table->write_set, field_index);
 }

=== modified file 'drizzled/filesort.cc'
--- drizzled/filesort.cc	2009-04-28 00:17:10 +0000
+++ drizzled/filesort.cc	2009-05-08 19:27:21 +0000
@@ -30,7 +30,6 @@
 #include <drizzled/table_list.h>
 
 #include <queue>
-#include <bitset>
 
 using namespace std;
 
@@ -454,7 +453,7 @@
   Session *session= current_session;
   volatile Session::killed_state *killed= &session->killed;
   handler *file;
-  bitset<MAX_FIELDS> *save_read_set, *save_write_set;
+  MY_BITMAP *save_read_set, *save_write_set;
 
   idx=indexpos=0;
   error=quick_select=0;
@@ -490,7 +489,7 @@
   save_read_set=  sort_form->read_set;
   save_write_set= sort_form->write_set;
   /* Set up temporary column read map for columns used by sort */
-  sort_form->tmp_set.reset();
+  bitmap_clear_all(&sort_form->tmp_set);
   /* Temporary set for register_used_fields and register_field_in_read_map */
   sort_form->read_set= &sort_form->tmp_set;
   register_used_fields(param);
@@ -921,7 +920,7 @@
 {
   register SORT_FIELD *sort_field;
   Table *table=param->sort_form;
-  bitset<MAX_FIELDS> *bitmap= table->read_set;
+  MY_BITMAP *bitmap= table->read_set;
 
   for (sort_field= param->local_sortorder ;
        sort_field != param->end ;
@@ -931,7 +930,7 @@
     if ((field= sort_field->field))
     {
       if (field->table == table)
-        bitmap->set(field->field_index);
+      bitmap_set_bit(bitmap, field->field_index);
     }
     else
     {						// Item
@@ -945,7 +944,7 @@
     SORT_ADDON_FIELD *addonf= param->addon_field;
     Field *field;
     for ( ; (field= addonf->field) ; addonf++)
-      bitmap->set(field->field_index);
+      bitmap_set_bit(bitmap, field->field_index);
   }
   else
   {
@@ -1455,7 +1454,7 @@
   uint32_t length= 0;
   uint32_t fields= 0;
   uint32_t null_fields= 0;
-  bitset<MAX_FIELDS> *read_set= (*ptabfield)->table->read_set;
+  MY_BITMAP *read_set= (*ptabfield)->table->read_set;
 
   /*
     If there is a reference to a field in the query add it
@@ -1470,7 +1469,7 @@
 
   for (pfield= ptabfield; (field= *pfield) ; pfield++)
   {
-    if (!read_set->test(field->field_index))
+    if (!bitmap_is_set(read_set, field->field_index))
       continue;
     if (field->flags & BLOB_FLAG)
       return 0;
@@ -1493,7 +1492,7 @@
   null_fields= 0;
   for (pfield= ptabfield; (field= *pfield) ; pfield++)
   {
-    if (!read_set->test(field->field_index))
+    if (!bitmap_is_set(read_set, field->field_index))
       continue;
     addonf->field= field;
     addonf->offset= length;

=== modified file 'drizzled/function/set_user_var.cc'
--- drizzled/function/set_user_var.cc	2009-04-28 00:17:10 +0000
+++ drizzled/function/set_user_var.cc	2009-05-08 19:27:21 +0000
@@ -24,10 +24,6 @@
 #include <drizzled/field/num.h>
 #include <drizzled/session.h>
 
-#include <bitset>
-
-using namespace std;
-
 /*
   When a user variable is updated (in a SET command or a query like
   SELECT @a:= ).
@@ -92,7 +88,7 @@
   {
     Table *table= (Table *) arg;
     if (result_field->table == table || !table)
-      result_field->table->read_set->set(result_field->field_index);
+      bitmap_set_bit(result_field->table->read_set, result_field->field_index);
   }
   return 0;
 }
@@ -104,11 +100,11 @@
 
 bool Item_func_set_user_var::register_field_in_bitmap(unsigned char *arg)
 {
-  bitset<MAX_FIELDS> *bitmap = (bitset<MAX_FIELDS> *) arg;
+  MY_BITMAP *bitmap = (MY_BITMAP *) arg;
   assert(bitmap);
   if (result_field)
   {
-    bitmap->set(result_field->field_index);
+    bitmap_set_bit(bitmap, result_field->field_index);
   }
   return 0;
 }

=== modified file 'drizzled/handler.cc'
--- drizzled/handler.cc	2009-05-08 19:07:39 +0000
+++ drizzled/handler.cc	2009-05-08 19:27:21 +0000
@@ -3562,7 +3562,11 @@
 */
 int handler::ha_reset()
 {
-  assert(table->s->all_set.size() == table->s->all_set.count());
+  /* Check that we have called all proper deallocation functions */
+  assert((unsigned char*) table->def_read_set.bitmap +
+              table->s->column_bitmap_size ==
+              (unsigned char*) table->def_write_set.bitmap);
+  assert(bitmap_is_set_all(&table->s->all_set));
   assert(table->key_read == 0);
   /* ensure that ha_index_end / ha_rnd_end has been called */
   assert(inited == NONE);

=== modified file 'drizzled/handler.h'
--- drizzled/handler.h	2009-05-08 19:07:39 +0000
+++ drizzled/handler.h	2009-05-08 19:27:21 +0000
@@ -37,10 +37,8 @@
 /* Bits to show what an alter table will do */
 #include <drizzled/sql_bitmap.h>
 
-#include<bitset>
-
 #define HA_MAX_ALTER_FLAGS 40
-typedef std::bitset<HA_MAX_ALTER_FLAGS> HA_ALTER_FLAGS;
+typedef Bitmap<HA_MAX_ALTER_FLAGS> HA_ALTER_FLAGS;
 
 
 typedef bool (*qc_engine_callback)(Session *session, char *table_key,
@@ -726,27 +724,6 @@
    check_if_incompatible_data(HA_CREATE_INFO *, uint32_t)
  { return COMPATIBLE_DATA_NO; }
 
- /*
-  * Check if the HA_ALTER_STORED_VCOL bit is set. If it is, then
-  * we will clear all other bits and return true; otherwise 
-  * clear all the bits in the the bitset and return false.
-  *
-  * @param    alter_flags    Bitmap that shows what will be changed
-  */
- virtual bool check_stored_vcol_flag(HA_ALTER_FLAGS *alter_flags)
- {
-   if (alter_flags->test(HA_ALTER_STORED_VCOL))
-   {
-     alter_flags->reset();
-     alter_flags->set(HA_ALTER_STORED_VCOL);
-   }
-   else
-   {
-     alter_flags->reset();
-   }
-   return (alter_flags->any());
- }
-
  /* On-line ALTER Table interface */
 
  /**
@@ -778,7 +755,7 @@
    if (this->check_if_incompatible_data(create_info, table_changes)
        == COMPATIBLE_DATA_NO)
      return(HA_ALTER_NOT_SUPPORTED);
-   else if (this->check_stored_vcol_flag(alter_flags))
+   else if ((*alter_flags & HA_ALTER_STORED_VCOL).is_set())
      return(HA_ALTER_NOT_SUPPORTED);
    else
      return(HA_ALTER_SUPPORTED_WAIT_LOCK);

=== modified file 'drizzled/item/field.cc'
--- drizzled/item/field.cc	2009-05-08 19:07:39 +0000
+++ drizzled/item/field.cc	2009-05-08 19:27:21 +0000
@@ -28,10 +28,6 @@
 #include <drizzled/item/field.h>
 #include <drizzled/item/outer_ref.h>
 
-#include <bitset>
-
-using namespace std;
-
 
 /**
   Store the pointer to this item field into a list if not already there.
@@ -109,7 +105,8 @@
 {
   Table *table= (Table *) arg;
   if (field->table == table || !table)
-    field->table->read_set->set(field->field_index);
+    bitmap_set_bit(field->table->read_set, field->field_index);
+
   return 0;
 }
 
@@ -120,9 +117,9 @@
 
 bool Item_field::register_field_in_bitmap(unsigned char *arg)
 {
-  bitset<MAX_FIELDS> *bitmap= (bitset<MAX_FIELDS> *) arg;
+  MY_BITMAP *bitmap= (MY_BITMAP *) arg;
   assert(bitmap);
-  bitmap->set(field->field_index);
+  bitmap_set_bit(bitmap, field->field_index);
   return false;
 }
 
@@ -724,20 +721,6 @@
   return 1;
 }
 
-/*
- * Helper function which tests whether a bit is set in the 
- * bitset or not. It also sets the bit after this test is
- * performed.
- */
-static bool test_and_set_bit(bitset<MAX_FIELDS> *bitmap, uint32_t pos)
-{
-  bool ret= false;
-  if (bitmap->test(pos))
-    ret= true;
-  bitmap->set(pos);
-  return ret;
-}
-
 
 /**
   Resolve the name of a column reference.
@@ -913,7 +896,7 @@
   else if (session->mark_used_columns != MARK_COLUMNS_NONE)
   {
     Table *table= field->table;
-    bitset<MAX_FIELDS> *current_bitmap, *other_bitmap;
+    MY_BITMAP *current_bitmap, *other_bitmap;
     if (session->mark_used_columns == MARK_COLUMNS_READ)
     {
       current_bitmap= table->read_set;
@@ -924,9 +907,9 @@
       current_bitmap= table->write_set;
       other_bitmap=   table->read_set;
     }
-    if (!test_and_set_bit(current_bitmap, field->field_index))
+    if (!bitmap_fast_test_and_set(current_bitmap, field->field_index))
     {
-      if (!other_bitmap->test(field->field_index))
+      if (!bitmap_is_set(other_bitmap, field->field_index))
       {
         /* First usage of column */
         table->used_fields++;                     // Used to optimize loops

=== modified file 'drizzled/key.cc'
--- drizzled/key.cc	2009-05-08 19:07:39 +0000
+++ drizzled/key.cc	2009-05-08 19:27:21 +0000
@@ -22,7 +22,6 @@
 #include "drizzled/field/blob.h"
 
 #include <string>
-#include <bitset>
 
 using namespace std;
 
@@ -390,11 +389,11 @@
     FALSE  Otherwise
 */
 
-bool is_key_used(Table *table, uint32_t idx, const bitset<MAX_FIELDS> *fields)
+bool is_key_used(Table *table, uint32_t idx, const MY_BITMAP *fields)
 {
-  table->tmp_set.reset();
+  bitmap_clear_all(&table->tmp_set);
   table->mark_columns_used_by_index_no_reset(idx, &table->tmp_set);
-  if (isBitmapOverlapping(fields, &table->tmp_set))
+  if (bitmap_is_overlapping(&table->tmp_set, fields))
     return 1;
 
   /*

=== modified file 'drizzled/key.h'
--- drizzled/key.h	2009-04-08 20:42:29 +0000
+++ drizzled/key.h	2009-05-08 19:27:21 +0000
@@ -27,7 +27,6 @@
 #include <drizzled/sql_list.h>
 #include <drizzled/lex_string.h>
 #include <drizzled/handler_structs.h>
-#include <bitset>
 
 class Item;
 typedef struct st_mem_root MEM_ROOT;
@@ -101,7 +100,7 @@
 void key_zero_nulls(unsigned char *tuple, KEY *key_info);
 bool key_cmp_if_same(Table *form,const unsigned char *key,uint32_t index,uint32_t key_length);
 void key_unpack(String *to,Table *form,uint32_t index);
-bool is_key_used(Table *table, uint32_t idx, const std::bitset<MAX_FIELDS> *fields);
+bool is_key_used(Table *table, uint32_t idx, const MY_BITMAP *fields);
 int key_cmp(KEY_PART_INFO *key_part, const unsigned char *key, uint32_t key_length);
 extern "C" int key_rec_cmp(void *key_info, unsigned char *a, unsigned char *b);
 #endif /* DRIZZLED_KEY_H */

=== modified file 'drizzled/opt_range.cc'
--- drizzled/opt_range.cc	2009-05-08 19:13:49 +0000
+++ drizzled/opt_range.cc	2009-05-08 19:27:21 +0000
@@ -114,7 +114,6 @@
 #include "drizzled/temporal.h" /* Needed in get_mm_leaf() for timestamp -> datetime comparisons */
 
 #include <string>
-#include <bitset>
 
 using namespace std;
 
@@ -699,8 +698,8 @@
   bool quick;				// Don't calulate possible keys
 
   uint32_t fields_bitmap_size;
-  bitset<MAX_FIELDS> needed_fields; /* bitmask of fields needed by the query */
-  bitset<MAX_FIELDS> tmp_covered_fields;
+  MY_BITMAP needed_fields;    /* bitmask of fields needed by the query */
+  MY_BITMAP tmp_covered_fields;
 
   key_map *needed_reg;        /* ptr to SQL_SELECT::needed_reg */
 
@@ -1091,9 +1090,12 @@
 {}
 
 QUICK_RANGE_SELECT::QUICK_RANGE_SELECT(Session *session, Table *table, uint32_t key_nr,
-                                       bool no_alloc, MEM_ROOT *parent_alloc)
+                                       bool no_alloc, MEM_ROOT *parent_alloc,
+                                       bool *create_error)
   :free_file(0),cur_range(NULL),last_range(0),dont_free(0)
 {
+  my_bitmap_map *bitmap;
+
   in_ror_merged_scan= 0;
   sorted= 0;
   index= key_nr;
@@ -1118,6 +1120,14 @@
   save_read_set= head->read_set;
   save_write_set= head->write_set;
 
+  /* Allocate a bitmap for used columns (Q: why not on MEM_ROOT?) */
+  if (!(bitmap= (my_bitmap_map*) malloc(head->s->column_bitmap_size)))
+  {
+    column_bitmap.bitmap= 0;
+    *create_error= 1;
+  }
+  else
+    bitmap_init(&column_bitmap, bitmap, head->s->fields, false);
   return;
 }
 
@@ -1159,6 +1169,7 @@
     }
     delete_dynamic(&ranges); /* ranges are allocated in alloc */
     free_root(&alloc,MYF(0));
+    free((char*) column_bitmap.bitmap);
   }
   head->column_bitmaps_set(save_read_set, save_write_set);
   if (mrr_buf_desc)
@@ -1344,7 +1355,7 @@
   }
   head->prepare_for_position();
   head->file= org_file;
-  column_bitmap= *(head->read_set);
+  bitmap_copy(&column_bitmap, head->read_set);
   head->column_bitmaps_set(&column_bitmap, &column_bitmap);
 
   return 0;
@@ -2085,11 +2096,17 @@
 static int fill_used_fields_bitmap(PARAM *param)
 {
   Table *table= param->table;
+  my_bitmap_map *tmp;
   uint32_t pk;
+  param->tmp_covered_fields.bitmap= 0;
   param->fields_bitmap_size= table->s->column_bitmap_size;
+  if (!(tmp= (my_bitmap_map*) alloc_root(param->mem_root,
+                                  param->fields_bitmap_size)) ||
+      bitmap_init(&param->needed_fields, tmp, table->s->fields, false))
+    return 1;
 
-  param->needed_fields = *(table->read_set);
-  param->needed_fields |= *(table->write_set);
+  bitmap_copy(&param->needed_fields, table->read_set);
+  bitmap_union(&param->needed_fields, table->write_set);
 
   pk= param->table->s->primary_key;
   if (pk != MAX_KEY && param->table->file->primary_key_is_clustered())
@@ -2099,7 +2116,7 @@
     KEY_PART_INFO *key_part_end= key_part +
                                  param->table->key_info[pk].key_parts;
     for (;key_part != key_part_end; ++key_part)
-      param->needed_fields.reset(key_part->fieldnr-1);
+      bitmap_clear_bit(&param->needed_fields, key_part->fieldnr-1);
   }
   return 0;
 }
@@ -2722,7 +2739,7 @@
   SEL_ARG   *sel_arg;
 
   /* Fields used in the query and covered by this ROR scan. */
-  bitset<MAX_FIELDS> covered_fields;
+  MY_BITMAP covered_fields;
   uint32_t      used_fields_covered; /* # of set bits in covered_fields */
   int       key_rec_length; /* length of key record (including rowid) */
 
@@ -2773,15 +2790,18 @@
                                                 param->fields_bitmap_size)))
     return NULL;
 
-  ror_scan->covered_fields.reset();
+  if (bitmap_init(&ror_scan->covered_fields, bitmap_buf,
+                  param->table->s->fields, false))
+    return NULL;
+  bitmap_clear_all(&ror_scan->covered_fields);
 
   KEY_PART_INFO *key_part= param->table->key_info[keynr].key_part;
   KEY_PART_INFO *key_part_end= key_part +
                                param->table->key_info[keynr].key_parts;
   for (;key_part != key_part_end; ++key_part)
   {
-    if (param->needed_fields.test(key_part->fieldnr-1))
-      ror_scan->covered_fields.set(key_part->fieldnr-1);
+    if (bitmap_is_set(&param->needed_fields, key_part->fieldnr-1))
+      bitmap_set_bit(&ror_scan->covered_fields, key_part->fieldnr-1);
   }
   double rows= rows2double(param->table->quick_rows[ror_scan->keynr]);
   ror_scan->index_read_cost=
@@ -2849,7 +2869,7 @@
 typedef struct
 {
   const PARAM *param;
-  bitset<MAX_FIELDS> covered_fields; /* union of fields covered by all scans */
+  MY_BITMAP covered_fields; /* union of fields covered by all scans */
   /*
     Fraction of table records that satisfies conditions of all scans.
     This is the number of full records that will be retrieved if a
@@ -2881,22 +2901,30 @@
 ROR_INTERSECT_INFO* ror_intersect_init(const PARAM *param)
 {
   ROR_INTERSECT_INFO *info;
+  my_bitmap_map* buf;
   if (!(info= (ROR_INTERSECT_INFO*)alloc_root(param->mem_root,
                                               sizeof(ROR_INTERSECT_INFO))))
     return NULL;
   info->param= param;
+  if (!(buf= (my_bitmap_map*) alloc_root(param->mem_root,
+                                         param->fields_bitmap_size)))
+    return NULL;
+  if (bitmap_init(&info->covered_fields, buf, param->table->s->fields,
+                  false))
+    return NULL;
   info->is_covering= false;
   info->index_scan_costs= 0.0;
   info->index_records= 0;
   info->out_rows= (double) param->table->file->stats.records;
-  info->covered_fields.reset();
+  bitmap_clear_all(&info->covered_fields);
   return info;
 }
 
 void ror_intersect_cpy(ROR_INTERSECT_INFO *dst, const ROR_INTERSECT_INFO *src)
 {
   dst->param= src->param;
-  dst->covered_fields= src->covered_fields;
+  memcpy(dst->covered_fields.bitmap, src->covered_fields.bitmap,
+         no_bytes_in_map(&src->covered_fields));
   dst->out_rows= src->out_rows;
   dst->is_covering= src->is_covering;
   dst->index_records= src->index_records;
@@ -3005,7 +3033,8 @@
   SEL_ARG *sel_arg, *tuple_arg= NULL;
   key_part_map keypart_map= 0;
   bool cur_covered;
-  bool prev_covered= test(info->covered_fields.test(key_part->fieldnr-1));
+  bool prev_covered= test(bitmap_is_set(&info->covered_fields,
+                                        key_part->fieldnr-1));
   key_range min_range;
   key_range max_range;
   min_range.key= key_val;
@@ -3017,7 +3046,8 @@
   for (sel_arg= scan->sel_arg; sel_arg;
        sel_arg= sel_arg->next_key_part)
   {
-    cur_covered= test(info->covered_fields.test(key_part[sel_arg->part].fieldnr-1));
+    cur_covered= test(bitmap_is_set(&info->covered_fields,
+                                    key_part[sel_arg->part].fieldnr-1));
     if (cur_covered != prev_covered)
     {
       /* create (part1val, ..., part{n-1}val) tuple. */
@@ -3129,9 +3159,9 @@
   {
     info->index_records += info->param->table->quick_rows[ror_scan->keynr];
     info->index_scan_costs += ror_scan->index_read_cost;
-    info->covered_fields |= ror_scan->covered_fields;
-    if (!info->is_covering && isBitmapSubset(&info->param->needed_fields,
-                                             &info->covered_fields))
+    bitmap_union(&info->covered_fields, &ror_scan->covered_fields);
+    if (!info->is_covering && bitmap_is_subset(&info->param->needed_fields,
+                                               &info->covered_fields))
     {
       info->is_covering= true;
     }
@@ -3366,6 +3396,7 @@
   return(trp);
 }
 
+
 /*
   Get best covering ROR-intersection.
   SYNOPSIS
@@ -3419,8 +3450,15 @@
   /*I=set of all covering indexes */
   ror_scan_mark= tree->ror_scans;
 
-  bitset<MAX_FIELDS> *covered_fields= &param->tmp_covered_fields;
-  covered_fields->reset();
+  MY_BITMAP *covered_fields= &param->tmp_covered_fields;
+  if (!covered_fields->bitmap)
+    covered_fields->bitmap= (my_bitmap_map*)alloc_root(param->mem_root,
+                                               param->fields_bitmap_size);
+  if (!covered_fields->bitmap ||
+      bitmap_init(covered_fields, covered_fields->bitmap,
+                  param->table->s->fields, false))
+    return 0;
+  bitmap_clear_all(covered_fields);
 
   double total_cost= 0.0f;
   ha_rows records=0;
@@ -3439,11 +3477,11 @@
     */
     for (ROR_SCAN_INFO **scan= ror_scan_mark; scan != ror_scans_end; ++scan)
     {
-      /* subtract a bitset */
-      (*scan)->covered_fields &= covered_fields->flip();
-      covered_fields->flip();
-      (*scan)->used_fields_covered= (*scan)->covered_fields.count();
-      (*scan)->first_uncovered_field= getFirstBitPos((*scan)->covered_fields);
+      bitmap_subtract(&(*scan)->covered_fields, covered_fields);
+      (*scan)->used_fields_covered=
+        bitmap_bits_set(&(*scan)->covered_fields);
+      (*scan)->first_uncovered_field=
+        bitmap_get_first(&(*scan)->covered_fields);
     }
 
     my_qsort(ror_scan_mark, ror_scans_end-ror_scan_mark, sizeof(ROR_SCAN_INFO*),
@@ -3459,8 +3497,8 @@
     if (total_cost > read_time)
       return NULL;
     /* F=F-covered by first(I) */
-    *covered_fields |= (*ror_scan_mark)->covered_fields;
-    all_covered= isBitmapSubset(&param->needed_fields, covered_fields);
+    bitmap_union(covered_fields, &(*ror_scan_mark)->covered_fields);
+    all_covered= bitmap_is_subset(&param->needed_fields, covered_fields);
   } while ((++ror_scan_mark < ror_scans_end) && !all_covered);
 
   if (!all_covered || (ror_scan_mark - tree->ror_scans) == 1)
@@ -6444,7 +6482,7 @@
 
   quick=new QUICK_RANGE_SELECT(param->session, param->table,
                                param->real_keynr[idx],
-                               test(parent_alloc), NULL);
+                               test(parent_alloc), NULL, &create_err);
 
   if (quick)
   {
@@ -6633,36 +6671,36 @@
 }
 
 
-bool QUICK_SELECT_I::is_keys_used(const bitset<MAX_FIELDS> *fields)
+bool QUICK_SELECT_I::is_keys_used(const MY_BITMAP *fields)
 {
   return is_key_used(head, index, fields);
 }
 
-bool QUICK_INDEX_MERGE_SELECT::is_keys_used(const bitset<MAX_FIELDS> *fields)
-{
-  QUICK_RANGE_SELECT *quick;
-  List_iterator_fast<QUICK_RANGE_SELECT> it(quick_selects);
-  while ((quick= it++))
-  {
-    if (is_key_used(head, quick->index, fields))
-      return 1;
-  }
-  return 0;
-}
-
-bool QUICK_ROR_INTERSECT_SELECT::is_keys_used(const bitset<MAX_FIELDS> *fields)
-{
-  QUICK_RANGE_SELECT *quick;
-  List_iterator_fast<QUICK_RANGE_SELECT> it(quick_selects);
-  while ((quick= it++))
-  {
-    if (is_key_used(head, quick->index, fields))
-      return 1;
-  }
-  return 0;
-}
-
-bool QUICK_ROR_UNION_SELECT::is_keys_used(const bitset<MAX_FIELDS> *fields)
+bool QUICK_INDEX_MERGE_SELECT::is_keys_used(const MY_BITMAP *fields)
+{
+  QUICK_RANGE_SELECT *quick;
+  List_iterator_fast<QUICK_RANGE_SELECT> it(quick_selects);
+  while ((quick= it++))
+  {
+    if (is_key_used(head, quick->index, fields))
+      return 1;
+  }
+  return 0;
+}
+
+bool QUICK_ROR_INTERSECT_SELECT::is_keys_used(const MY_BITMAP *fields)
+{
+  QUICK_RANGE_SELECT *quick;
+  List_iterator_fast<QUICK_RANGE_SELECT> it(quick_selects);
+  while ((quick= it++))
+  {
+    if (is_key_used(head, quick->index, fields))
+      return 1;
+  }
+  return 0;
+}
+
+bool QUICK_ROR_UNION_SELECT::is_keys_used(const MY_BITMAP *fields)
 {
   QUICK_SELECT_I *quick;
   List_iterator_fast<QUICK_SELECT_I> it(quick_selects);
@@ -6708,7 +6746,7 @@
 
   old_root= session->mem_root;
   /* The following call may change session->mem_root */
-  quick= new QUICK_RANGE_SELECT(session, table, ref->key, 0, 0);
+  quick= new QUICK_RANGE_SELECT(session, table, ref->key, 0, 0, &create_err);
   /* save mem_root set by QUICK_RANGE_SELECT constructor */
   alloc= session->mem_root;
   /*
@@ -8063,7 +8101,7 @@
           If the field is used in the current query ensure that it's
           part of 'cur_index'
         */
-        if (table->read_set->test(cur_field->field_index) &&
+        if (bitmap_is_set(table->read_set, cur_field->field_index) &&
             !cur_field->part_of_key_not_clustered.is_set(cur_index))
           goto next_index;                  // Field was not part of key
       }
@@ -8235,7 +8273,7 @@
                 (min_max_arg_part && (min_max_arg_part < last_part));
       for (; cur_part != last_part; cur_part++)
       {
-        if (table->read_set->test(cur_part->field->field_index))
+        if (bitmap_is_set(table->read_set, cur_part->field->field_index))
           goto next_index;
       }
     }

=== modified file 'drizzled/opt_range.h'
--- drizzled/opt_range.h	2009-04-08 23:01:10 +0000
+++ drizzled/opt_range.h	2009-05-08 19:27:21 +0000
@@ -26,7 +26,6 @@
 #include <drizzled/field.h>
 #include <drizzled/item/sum.h>
 #include <queue>
-#include <bitset>
 
 class JOIN;
 typedef class Item COND;
@@ -249,7 +248,7 @@
     Return 1 if any index used by this quick select
     uses field which is marked in passed bitmap.
   */
-  virtual bool is_keys_used(const std::bitset<MAX_FIELDS> *fields);
+  virtual bool is_keys_used(const MY_BITMAP *fields);
 
   /*
     rowid of last row retrieved by this quick select. This is used only when
@@ -296,7 +295,7 @@
 
   /* Members to deal with case when this quick select is a ROR-merged scan */
   bool in_ror_merged_scan;
-  std::bitset<MAX_FIELDS> column_bitmap, *save_read_set, *save_write_set;
+  MY_BITMAP column_bitmap, *save_read_set, *save_write_set;
   bool free_file;   /* TRUE <=> this->file is "owned" by this quick select */
 
   /* Range pointers to be used when not using MRR interface */
@@ -324,7 +323,7 @@
   MEM_ROOT alloc;
 
   QUICK_RANGE_SELECT(Session *session, Table *table,uint32_t index_arg,bool no_alloc,
-                     MEM_ROOT *parent_alloc);
+                     MEM_ROOT *parent_alloc, bool *create_err);
   ~QUICK_RANGE_SELECT();
 
   int init();
@@ -452,7 +451,7 @@
   int get_type() { return QS_TYPE_INDEX_MERGE; }
   void add_keys_and_lengths(String *key_names, String *used_lengths);
   void add_info_string(String *str);
-  bool is_keys_used(const std::bitset<MAX_FIELDS> *fields);
+  bool is_keys_used(const MY_BITMAP *fields);
 
   bool push_quick_back(QUICK_RANGE_SELECT *quick_sel_range);
 
@@ -508,7 +507,7 @@
   int get_type() { return QS_TYPE_ROR_INTERSECT; }
   void add_keys_and_lengths(String *key_names, String *used_lengths);
   void add_info_string(String *str);
-  bool is_keys_used(const std::bitset<MAX_FIELDS> *fields);
+  bool is_keys_used(const MY_BITMAP *fields);
   int init_ror_merged_scan(bool reuse_handler);
   bool push_quick_back(QUICK_RANGE_SELECT *quick_sel_range);
 
@@ -567,7 +566,7 @@
   int get_type() { return QS_TYPE_ROR_UNION; }
   void add_keys_and_lengths(String *key_names, String *used_lengths);
   void add_info_string(String *str);
-  bool is_keys_used(const std::bitset<MAX_FIELDS> *fields);
+  bool is_keys_used(const MY_BITMAP *fields);
 
   bool push_quick_back(QUICK_SELECT_I *quick_sel_range);
 

=== modified file 'drizzled/server_includes.h'
--- drizzled/server_includes.h	2009-05-08 19:13:49 +0000
+++ drizzled/server_includes.h	2009-05-08 19:27:21 +0000
@@ -161,7 +161,7 @@
 extern pthread_cond_t COND_global_read_lock;
 extern pthread_attr_t connection_attrib;
 extern I_List<Session> session_list;
-extern ThreadSafeBitset temp_pool;
+extern MY_BITMAP temp_pool;
 extern String my_empty_string;
 extern const String my_null_string;
 extern SHOW_VAR status_vars[];

=== modified file 'drizzled/show.cc'
--- drizzled/show.cc	2009-05-08 19:07:39 +0000
+++ drizzled/show.cc	2009-05-08 19:27:21 +0000
@@ -617,7 +617,7 @@
   TableShare *share= table->s;
   HA_CREATE_INFO create_info;
   bool show_table_options= false;
-  bitset<MAX_FIELDS> *old_bitmap;
+  my_bitmap_map *old_map;
 
   table->restoreRecordAsDefault(); // Get empty record
 
@@ -644,7 +644,7 @@
     We have to restore the read_set if we are called from insert in case
     of row based replication.
   */
-  old_bitmap= table->use_all_columns(table->read_set);
+  old_map= table->use_all_columns(table->read_set);
 
   for (ptr=table->field ; (field= *ptr); ptr++)
   {
@@ -901,7 +901,7 @@
     append_directory(session, packet, "DATA",  create_info.data_file_name);
     append_directory(session, packet, "INDEX", create_info.index_file_name);
   }
-  table->restore_column_map(old_bitmap);
+  table->restore_column_map(old_map);
   return(0);
 }
 
@@ -2297,7 +2297,7 @@
     int table_open_method= 0, field_indx= 0;
     for (ptr=tables->table->field; (field= *ptr) ; ptr++)
     {
-      if (tables->table->read_set->test(field->field_index))
+      if (bitmap_is_set(tables->table->read_set, field->field_index))
         table_open_method|= schema_table->fields_info[field_indx].open_method;
       field_indx++;
     }
@@ -3020,10 +3020,16 @@
     if (!show_table->read_set)
     {
       /* to satisfy 'field->val_str' ASSERTs */
-      show_table->def_read_set.set();
+      unsigned char *bitmaps;
+      uint32_t bitmap_size= show_table_share->column_bitmap_size;
+      if (!(bitmaps= (unsigned char*) alloc_root(session->mem_root, bitmap_size)))
+        return(0);
+      bitmap_init(&show_table->def_read_set,
+                  (my_bitmap_map*) bitmaps, show_table_share->fields, false);
+      bitmap_set_all(&show_table->def_read_set);
       show_table->read_set= &show_table->def_read_set;
     }
-    show_table->read_set->set();
+    bitmap_set_all(show_table->read_set);
   }
 
   for (; (field= *ptr) ; ptr++)
@@ -3793,9 +3799,12 @@
                                  TMP_TABLE_ALL_COLUMNS),
                                 HA_POS_ERROR, table_list->alias)))
     return(0);
-  table->def_read_set.set();
+  my_bitmap_map* bitmaps=
+    (my_bitmap_map*) session->alloc(bitmap_buffer_size(field_count));
+  bitmap_init(&table->def_read_set, (my_bitmap_map*) bitmaps, field_count,
+              false);
   table->read_set= &table->def_read_set;
-  table->read_set->reset();
+  bitmap_clear_all(table->read_set);
   table_list->schema_table_param= tmp_table_param;
   return(table);
 }

=== modified file 'drizzled/sj_tmp_table.cc'
--- drizzled/sj_tmp_table.cc	2009-05-08 19:13:49 +0000
+++ drizzled/sj_tmp_table.cc	2009-05-08 19:27:21 +0000
@@ -81,7 +81,7 @@
   */
   statistic_increment(session->status_var.created_tmp_tables, &LOCK_status);
   if (use_temp_pool && !(test_flags & TEST_KEEP_TMP_TABLES))
-    temp_pool_slot = temp_pool.setNextBit();
+    temp_pool_slot = bitmap_lock_set_next(&temp_pool);
 
   if (temp_pool_slot != MY_BIT_NONE) // we got a slot
     sprintf(path, "%s_%lx_%i", TMP_FILE_PREFIX,
@@ -116,7 +116,7 @@
                         NULL))
   {
     if (temp_pool_slot != MY_BIT_NONE)
-      temp_pool.resetBit(temp_pool_slot);
+      bitmap_lock_clear_bit(&temp_pool, temp_pool_slot);
     return(NULL);
   }
   strcpy(tmpname,path);
@@ -214,7 +214,7 @@
     table->record[1]= table->record[0]+alloc_length;
     share->default_values= table->record[1]+alloc_length;
   }
-  table->setup_tmp_table_column_bitmaps();
+  table->setup_tmp_table_column_bitmaps(bitmaps);
 
   recinfo= start_recinfo;
   null_flags=(unsigned char*) table->record[0];
@@ -328,6 +328,6 @@
   session->mem_root= mem_root_save;
   table->free_tmp_table(session);                    /* purecov: inspected */
   if (temp_pool_slot != MY_BIT_NONE)
-    temp_pool.resetBit(temp_pool_slot);
+    bitmap_lock_clear_bit(&temp_pool, temp_pool_slot);
   return(NULL);        /* purecov: inspected */
 }

=== modified file 'drizzled/sql_base.cc'
--- drizzled/sql_base.cc	2009-05-08 19:07:39 +0000
+++ drizzled/sql_base.cc	2009-05-08 19:27:21 +0000
@@ -45,10 +45,6 @@
 #include <drizzled/check_stack_overrun.h>
 #include <drizzled/lock.h>
 
-#include <bitset>
-
-using namespace std;
-
 extern drizzled::TransactionServices transaction_services;
 
 /**
@@ -3523,19 +3519,6 @@
   return(error);
 }
 
-/*
- * Helper function which tests whether a bit is set in the 
- * bitset or not. It also sets the bit after this test is
- * performed.
- */
-static bool test_and_set_bit(bitset<MAX_FIELDS> *bitmap, uint32_t pos)
-{
-  bool ret= false;
-  if (bitmap->test(pos))
-    ret= true;
-  bitmap->set(pos);
-  return ret;
-}
 
 /*****************************************************************************
 * The following find_field_in_XXX procedures implement the core of the
@@ -3556,7 +3539,7 @@
 {
   if (session->mark_used_columns != MARK_COLUMNS_NONE)
   {
-    bitset<MAX_FIELDS> *current_bitmap, *other_bitmap;
+    MY_BITMAP *current_bitmap, *other_bitmap;
 
     /*
       We always want to register the used keys, as the column bitmap may have
@@ -3577,7 +3560,7 @@
       other_bitmap=   table->read_set;
     }
 
-    if (test_and_set_bit(current_bitmap, field->field_index))
+    if (bitmap_fast_test_and_set(current_bitmap, field->field_index))
     {
       if (session->mark_used_columns == MARK_COLUMNS_WRITE)
         session->dup_field= field;
@@ -3894,9 +3877,9 @@
         {
           Table *table= field_to_set->table;
           if (session->mark_used_columns == MARK_COLUMNS_READ)
-            table->read_set->set(field_to_set->field_index);
+            bitmap_set_bit(table->read_set, field_to_set->field_index);
           else
-            table->write_set->set(field_to_set->field_index);
+            bitmap_set_bit(table->write_set, field_to_set->field_index);
         }
       }
   }
@@ -4644,7 +4627,7 @@
       {
         Table *table_1= nj_col_1->table_ref->table;
         /* Mark field_1 used for table cache. */
-        table_1->read_set->set(field_1->field_index);
+        bitmap_set_bit(table_1->read_set, field_1->field_index);
         table_1->covering_keys.intersect(field_1->part_of_key);
         table_1->merge_keys.merge(field_1->part_of_key);
       }
@@ -4652,7 +4635,7 @@
       {
         Table *table_2= nj_col_2->table_ref->table;
         /* Mark field_2 used for table cache. */
-        table_2->read_set->set(field_2->field_index);
+        bitmap_set_bit(table_2->read_set, field_2->field_index);
         table_2->covering_keys.intersect(field_2->part_of_key);
         table_2->merge_keys.merge(field_2->part_of_key);
       }
@@ -5446,7 +5429,7 @@
       if ((field= field_iterator.field()))
       {
         /* Mark fields as used to allow storage engine to optimze access */
-        field->table->read_set->set(field->field_index);
+        bitmap_set_bit(field->table->read_set, field->field_index);
         if (table)
         {
           table->covering_keys.intersect(field->part_of_key);

=== removed file 'drizzled/sql_bitmap.cc'
--- drizzled/sql_bitmap.cc	2009-05-08 19:13:49 +0000
+++ drizzled/sql_bitmap.cc	1970-01-01 00:00:00 +0000
@@ -1,80 +0,0 @@
-/* - mode: c; c-basic-offset: 2; indent-tabs-mode: nil; -*-
- *  vim:expandtab:shiftwidth=2:tabstop=2:smarttab:
- *
- *  Copyright (C) 2009 Sun Microsystems
- *
- *  Authors:
- *
- *  Padraig O'Sullivan <osullivan.padraig@gmail.com>
- *
- *  This program is free software; you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation; either version 2 of the License, or
- *  (at your option) any later version.
- *
- *  This program is distributed in the hope that it will be useful,
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU General Public License for more details.
- *
- *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
- */
-
-/**
- * @file 
- *
- * Implementation of helper methods for working with
- * bitsets and methods for thread-safe bitsets.
- */
-
-#include "drizzled/sql_bitmap.h"
-
-#include <bitset>
-
-using namespace std;
-
-void ThreadSafeBitset::resetBit(uint32_t pos)
-{
-  pthread_mutex_lock(&mutex);
-  bitmap.reset(pos);
-  pthread_mutex_unlock(&mutex);
-}
-
-uint32_t ThreadSafeBitset::setNextBit()
-{
-  uint32_t bit_found;
-  pthread_mutex_lock(&mutex);
-  if ((bit_found= getFirstBitPos(bitmap)) != MY_BIT_NONE)
-    bitmap.set(bit_found);
-  pthread_mutex_unlock(&mutex);
-  return bit_found;
-}
-
-uint32_t getFirstBitPos(const bitset<MAX_FIELDS> &bitmap)
-{
-  uint32_t first_bit= MY_BIT_NONE;
-  for (int idx= 0; idx < MAX_FIELDS; idx++)
-  {
-    if (!bitmap.test(idx))
-    {
-      first_bit= idx;
-      break;
-    }
-  }
-  return first_bit;
-}
-
-bool isBitmapSubset(const bitset<MAX_FIELDS> *map1, const bitset<MAX_FIELDS> *map2)
-{
-  bitset<MAX_FIELDS> tmp1= *map2;
-  bitset<MAX_FIELDS> tmp2= *map1 & tmp1.flip();
-  return (!tmp2.any());
-}
-
-bool isBitmapOverlapping(const bitset<MAX_FIELDS> *map1, const bitset<MAX_FIELDS> *map2)
-{
-  bitset<MAX_FIELDS> tmp= *map1 & *map2;
-  return (tmp.any());
-}

=== modified file 'drizzled/sql_bitmap.h'
--- drizzled/sql_bitmap.h	2009-05-08 19:13:49 +0000
+++ drizzled/sql_bitmap.h	2009-05-08 19:27:21 +0000
@@ -29,11 +29,8 @@
 /// TODO: OMG FIX THIS
 
 #include <mysys/my_bitmap.h>
-#include <drizzled/definitions.h>
 #include <drizzled/util/test.h>
 
-#include <bitset>
-
 template <uint32_t default_width> class Bitmap
 {
   MY_BITMAP map;
@@ -218,63 +215,4 @@
 extern const key_map key_map_empty;
 extern key_map key_map_full;          /* Should be threaded as const */
 
-/*
- * Class to be used when a thread safe version of std::bitset
- * is needed. We just use a lock here to protect any modifications
- * to the bitset.
- */
-class ThreadSafeBitset
-{
-public:
-  ThreadSafeBitset()
-  {
-    pthread_mutex_init(&mutex, NULL);
-  }
-  ~ThreadSafeBitset()
-  {
-    pthread_mutex_destroy(&mutex);
-  }
-  /*
-   * Resets a bit at the given bit position.
-   *
-   * @param position of the bit to reset
-   */
-  void resetBit(uint32_t pos);
-  /*
-   * Finds the first bit that is not set and sets
-   * it.
-   */
-  uint32_t setNextBit();
-private:
-  std::bitset<MAX_FIELDS> bitmap;
-  pthread_mutex_t mutex;
-};
-
-/*
- * Returns the position of the first bit in the
- * given bitmap which is not set. If every bit is set
- * in the bitmap, return MY_BIT_NONE.
- *
- * @param the bitmap to work with
- */
-uint32_t getFirstBitPos(const std::bitset<MAX_FIELDS> &bitmap);
-
-/*
- * Returns true if map1 is a subset of map2; otherwise,
- * it returns false.
- *
- * @param the bitmap to check with
- * @param the bitmap to check against
- */
-bool isBitmapSubset(const std::bitset<MAX_FIELDS> *map1, const std::bitset<MAX_FIELDS> *map2);
-
-/*
- * Returns true if there is any overlapping bits between
- * the 2 given bitmaps.
- *
- * @param the first bitmap to work with
- * @param the second bitmap to work with
- */
-bool isBitmapOverlapping(const std::bitset<MAX_FIELDS> *map1, const std::bitset<MAX_FIELDS> *map2);
-
 #endif /* _SQL_BITMAP_H_ */

=== modified file 'drizzled/sql_insert.cc'
--- drizzled/sql_insert.cc	2009-05-08 19:13:49 +0000
+++ drizzled/sql_insert.cc	2009-05-08 19:27:21 +0000
@@ -33,10 +33,6 @@
 #include <drizzled/field/timestamp.h>
 #include <drizzled/lock.h>
 
-#include <bitset>
-
-using namespace std;
-
 
 /*
   Check if insert fields are correct.
@@ -79,7 +75,7 @@
       No fields are provided so all fields must be provided in the values.
       Thus we set all bits in the write set.
     */
-    table->write_set->set();
+    bitmap_set_all(table->write_set);
   }
   else
   {						// Part field list
@@ -120,12 +116,14 @@
     }
     if (table->timestamp_field)	// Don't automaticly set timestamp if used
     {
-      if (table->write_set->test(table->timestamp_field->field_index))
+      if (bitmap_is_set(table->write_set,
+                        table->timestamp_field->field_index))
         clear_timestamp_auto_bits(table->timestamp_field_type,
                                   TIMESTAMP_AUTO_SET_ON_INSERT);
       else
       {
-        table->write_set->set(table->timestamp_field->field_index);
+        bitmap_set_bit(table->write_set,
+                       table->timestamp_field->field_index);
       }
     }
   }
@@ -166,8 +164,8 @@
       Unmark the timestamp field so that we can check if this is modified
       by update_fields
     */
-    timestamp_mark= table->write_set->test(table->timestamp_field->field_index);
-    table->write_set->reset(table->timestamp_field->field_index);
+    timestamp_mark= bitmap_test_and_clear(table->write_set,
+                                          table->timestamp_field->field_index);
   }
 
   /* Check the fields we are going to modify */
@@ -177,11 +175,13 @@
   if (table->timestamp_field)
   {
     /* Don't set timestamp column if this is modified. */
-    if (table->write_set->test(table->timestamp_field->field_index))
+    if (bitmap_is_set(table->write_set,
+                      table->timestamp_field->field_index))
       clear_timestamp_auto_bits(table->timestamp_field_type,
                                 TIMESTAMP_AUTO_SET_ON_UPDATE);
     if (timestamp_mark)
-      table->write_set->set(table->timestamp_field->field_index);
+      bitmap_set_bit(table->write_set,
+                     table->timestamp_field->field_index);
   }
   return 0;
 }
@@ -718,7 +718,7 @@
 {
   int error;
   char *key=0;
-  bitset<MAX_FIELDS> *save_read_set, *save_write_set;
+  MY_BITMAP *save_read_set, *save_write_set;
   uint64_t prev_insert_id= table->file->next_insert_id;
   uint64_t insert_id_for_cur_row= 0;
 
@@ -825,7 +825,7 @@
             table->next_number_field->val_int());
         info->touched++;
         if ((table->file->ha_table_flags() & HA_PARTIAL_COLUMN_READ &&
-             !isBitmapSubset(table->write_set, table->read_set)) ||
+             !bitmap_is_subset(table->write_set, table->read_set)) ||
             table->compare_record())
         {
           if ((error=table->file->ha_update_row(table->record[1],
@@ -962,11 +962,11 @@
                                            TableList *)
 {
   int err= 0;
-  bitset<MAX_FIELDS> *write_set= entry->write_set;
+  MY_BITMAP *write_set= entry->write_set;
 
   for (Field **field=entry->field ; *field ; field++)
   {
-    if (!write_set->test((*field)->field_index))
+    if (!bitmap_is_set(write_set, (*field)->field_index))
     {
       /*
        * If the field doesn't have any default value
@@ -1663,7 +1663,7 @@
 
   /* Mark all fields that are given values */
   for (Field **f= field ; *f ; f++)
-    table->write_set->set((*f)->field_index);
+    bitmap_set_bit(table->write_set, (*f)->field_index);
 
   /* Don't set timestamp if used */
   table->timestamp_field_type= TIMESTAMP_NO_AUTO_SET;

=== modified file 'drizzled/sql_load.cc'
--- drizzled/sql_load.cc	2009-04-25 00:00:44 +0000
+++ drizzled/sql_load.cc	2009-05-08 19:27:21 +0000
@@ -166,7 +166,7 @@
     Field **field;
     for (field=table->field; *field ; field++)
       fields_vars.push_back(new Item_field(*field));
-    table->write_set->set();
+    bitmap_set_all(table->write_set);
     table->timestamp_field_type= TIMESTAMP_NO_AUTO_SET;
     /*
       Let us also prepare SET clause, altough it is probably empty
@@ -189,11 +189,13 @@
     */
     if (table->timestamp_field)
     {
-      if (table->write_set->test(table->timestamp_field->field_index))
+      if (bitmap_is_set(table->write_set,
+                        table->timestamp_field->field_index))
         table->timestamp_field_type= TIMESTAMP_NO_AUTO_SET;
       else
       {
-        table->write_set->set(table->timestamp_field->field_index);
+        bitmap_set_bit(table->write_set,
+                       table->timestamp_field->field_index);
       }
     }
     /* Fix the expressions in SET clause */

=== modified file 'drizzled/sql_select.cc'
--- drizzled/sql_select.cc	2009-05-08 19:07:39 +0000
+++ drizzled/sql_select.cc	2009-05-08 19:27:21 +0000
@@ -49,7 +49,6 @@
 #include "drizzled/index_hint.h"
 
 #include <string>
-#include <bitset>
 
 using namespace std;
 
@@ -6229,12 +6228,12 @@
 {
   uint32_t null_fields,blobs,fields,rec_length;
   Field **f_ptr,*field;
-  bitset<MAX_FIELDS> *read_set= join_tab->table->read_set;
+  MY_BITMAP *read_set= join_tab->table->read_set;;
 
   null_fields= blobs= fields= rec_length=0;
   for (f_ptr=join_tab->table->field ; (field= *f_ptr) ; f_ptr++)
   {
-    if (read_set->test(field->field_index))
+    if (bitmap_is_set(read_set, field->field_index))
     {
       uint32_t flags=field->flags;
       fields++;
@@ -6255,9 +6254,9 @@
 			     (join_tab->table->getRecordLength()- rec_length));
     rec_length+=(uint32_t) cmax((uint32_t)4,blob_length);
   }
-  join_tab->used_fields= fields;
-  join_tab->used_fieldlength= rec_length;
-  join_tab->used_blobs= blobs;
+  join_tab->used_fields=fields;
+  join_tab->used_fieldlength=rec_length;
+  join_tab->used_blobs=blobs;
 }
 
 
@@ -13753,13 +13752,13 @@
   {
     uint32_t null_fields=0, used_fields;
     Field **f_ptr,*field;
-    bitset<MAX_FIELDS> *read_set= tables[i].table->read_set;
+    MY_BITMAP *read_set= tables[i].table->read_set;
     for (f_ptr=tables[i].table->field,used_fields=tables[i].used_fields ;
 	 used_fields ;
 	 f_ptr++)
     {
       field= *f_ptr;
-      if (read_set->test(field->field_index))
+      if (bitmap_is_set(read_set, field->field_index))
       {
 	used_fields--;
 	length+=field->fill_cache_field(copy);
@@ -13821,7 +13820,7 @@
             (size_t)cache->length);
   if (!(cache->buff=(unsigned char*) malloc(size)))
     return 1;				/* Don't use cache */ /* purecov: inspected */
-  cache->end= cache->buff+size;
+  cache->end=cache->buff+size;
   reset_cache_write(cache);
   return 0;
 }

=== modified file 'drizzled/sql_table.cc'
--- drizzled/sql_table.cc	2009-05-08 19:07:39 +0000
+++ drizzled/sql_table.cc	2009-05-08 19:27:21 +0000
@@ -3165,29 +3165,29 @@
   uint32_t flags= alter_info->flags;
 
   if (ALTER_ADD_COLUMN & flags)
-    alter_flags->set(HA_ADD_COLUMN);
+    *alter_flags|= HA_ADD_COLUMN;
   if (ALTER_DROP_COLUMN & flags)
-    alter_flags->set(HA_DROP_COLUMN);
+    *alter_flags|= HA_DROP_COLUMN;
   if (ALTER_RENAME & flags)
-    alter_flags->set(HA_RENAME_TABLE);
+    *alter_flags|= HA_RENAME_TABLE;
   if (ALTER_CHANGE_COLUMN & flags)
-    alter_flags->set(HA_CHANGE_COLUMN);
+    *alter_flags|= HA_CHANGE_COLUMN;
   if (ALTER_COLUMN_DEFAULT & flags)
-    alter_flags->set(HA_COLUMN_DEFAULT_VALUE);
+    *alter_flags|= HA_COLUMN_DEFAULT_VALUE;
   if (ALTER_COLUMN_STORAGE & flags)
-    alter_flags->set(HA_COLUMN_STORAGE);
+    *alter_flags|= HA_COLUMN_STORAGE;
   if (ALTER_COLUMN_FORMAT & flags)
-    alter_flags->set(HA_COLUMN_FORMAT);
+    *alter_flags|= HA_COLUMN_FORMAT;
   if (ALTER_COLUMN_ORDER & flags)
-    alter_flags->set(HA_ALTER_COLUMN_ORDER);
+    *alter_flags|= HA_ALTER_COLUMN_ORDER;
   if (ALTER_STORAGE & flags)
-    alter_flags->set(HA_ALTER_STORAGE);
+    *alter_flags|= HA_ALTER_STORAGE;
   if (ALTER_ROW_FORMAT & flags)
-    alter_flags->set(HA_ALTER_ROW_FORMAT);
+    *alter_flags|= HA_ALTER_ROW_FORMAT;
   if (ALTER_RECREATE & flags)
-    alter_flags->set(HA_RECREATE);
+    *alter_flags|= HA_RECREATE;
   if (ALTER_FOREIGN_KEY & flags)
-    alter_flags->set(HA_ALTER_FOREIGN_KEY);
+    *alter_flags|= HA_ALTER_FOREIGN_KEY;
 }
 
 
@@ -3326,21 +3326,21 @@
       Check what has changed and set alter_flags
     */
     if (table->s->fields < alter_info->create_list.elements)
-      alter_flags->set(HA_ADD_COLUMN);
+      *alter_flags|= HA_ADD_COLUMN;
     else if (table->s->fields > alter_info->create_list.elements)
-      alter_flags->set(HA_DROP_COLUMN);
+      *alter_flags|= HA_DROP_COLUMN;
     if (create_info->db_type != table->s->db_type() ||
         create_info->used_fields & HA_CREATE_USED_ENGINE)
-      alter_flags->set(HA_ALTER_STORAGE_ENGINE);
+      *alter_flags|= HA_ALTER_STORAGE_ENGINE;
     if (create_info->used_fields & HA_CREATE_USED_CHARSET)
-      alter_flags->set(HA_CHANGE_CHARACTER_SET);
+      *alter_flags|= HA_CHANGE_CHARACTER_SET;
     if (create_info->used_fields & HA_CREATE_USED_DEFAULT_CHARSET)
-      alter_flags->set(HA_SET_DEFAULT_CHARACTER_SET);
+      *alter_flags|= HA_SET_DEFAULT_CHARACTER_SET;
     if (alter_info->flags & ALTER_RECREATE)
-      alter_flags->set(HA_RECREATE);
+      *alter_flags|= HA_RECREATE;
     /* TODO check for ADD/DROP FOREIGN KEY */
     if (alter_info->flags & ALTER_FOREIGN_KEY)
-      alter_flags->set(HA_ALTER_FOREIGN_KEY);
+      *alter_flags|=  HA_ALTER_FOREIGN_KEY;
   }
   /*
     Go through fields and check if the original ones are compatible
@@ -3365,7 +3365,7 @@
     {
       /* Evaluate changes bitmap and send to check_if_incompatible_data() */
       if (!(table_changes_local= field->is_equal(new_field)))
-        alter_flags->set(HA_ALTER_COLUMN_TYPE);
+        *alter_flags|= HA_ALTER_COLUMN_TYPE;
 
       /* Check if field was renamed */
       field->flags&= ~FIELD_IS_RENAMED;
@@ -3374,19 +3374,19 @@
                         new_field->field_name))
       {
         field->flags|= FIELD_IS_RENAMED;
-        alter_flags->set(HA_ALTER_COLUMN_NAME);
+        *alter_flags|= HA_ALTER_COLUMN_NAME;
       }
 
       *table_changes&= table_changes_local;
       if (table_changes_local == IS_EQUAL_PACK_LENGTH)
-        alter_flags->set(HA_ALTER_COLUMN_TYPE);
+        *alter_flags|= HA_ALTER_COLUMN_TYPE;
 
       /* Check that NULL behavior is same for old and new fields */
       if ((new_field->flags & NOT_NULL_FLAG) !=
           (uint32_t) (field->flags & NOT_NULL_FLAG))
       {
         *table_changes= IS_EQUAL_NO;
-        alter_flags->set(HA_ALTER_COLUMN_NULLABLE);
+        *alter_flags|= HA_ALTER_COLUMN_NULLABLE;
       }
     }
 
@@ -3433,12 +3433,12 @@
       {
         /* Unique key. Check for "PRIMARY". */
         if (is_primary_key(table_key))
-          alter_flags->set(HA_DROP_PK_INDEX);
+          *alter_flags|= HA_DROP_PK_INDEX;
         else
-          alter_flags->set(HA_DROP_UNIQUE_INDEX);
+          *alter_flags|= HA_DROP_UNIQUE_INDEX;
       }
       else
-        alter_flags->set(HA_DROP_INDEX);
+        *alter_flags|= HA_DROP_INDEX;
       *table_changes= IS_EQUAL_NO;
       continue;
     }
@@ -3453,12 +3453,12 @@
       {
         // Unique key. Check for "PRIMARY".
         if (is_primary_key(table_key))
-          alter_flags->set(HA_ALTER_PK_INDEX);
+          *alter_flags|= HA_ALTER_PK_INDEX;
         else
-          alter_flags->set(HA_ALTER_UNIQUE_INDEX);
+          *alter_flags|= HA_ALTER_UNIQUE_INDEX;
       }
       else
-        alter_flags->set(HA_ALTER_INDEX);
+        *alter_flags|= HA_ALTER_INDEX;
       goto index_changed;
     }
 
@@ -3482,12 +3482,12 @@
         {
           /* Unique key. Check for "PRIMARY" */
           if (is_primary_key(table_key))
-            alter_flags->set(HA_ALTER_PK_INDEX);
+            *alter_flags|= HA_ALTER_PK_INDEX;
           else
-            alter_flags->set(HA_ALTER_UNIQUE_INDEX);
+            *alter_flags|= HA_ALTER_UNIQUE_INDEX;
         }
         else
-          alter_flags->set(HA_ALTER_INDEX);
+          *alter_flags|= HA_ALTER_INDEX;
         goto index_changed;
       }
     }
@@ -3544,12 +3544,12 @@
       {
         /* Unique key. Check for "PRIMARY" */
         if (is_primary_key(new_key))
-          alter_flags->set(HA_ADD_PK_INDEX);
+          *alter_flags|= HA_ADD_PK_INDEX;
         else
-        alter_flags->set(HA_ADD_UNIQUE_INDEX);
+        *alter_flags|= HA_ADD_UNIQUE_INDEX;
       }
       else
-        alter_flags->set(HA_ADD_INDEX);
+        *alter_flags|= HA_ADD_INDEX;
       *table_changes= IS_EQUAL_NO;
     }
   }
@@ -4643,7 +4643,7 @@
       can do the change on-line
     */
     if (new_name == table_name && new_db == db &&
-        ha_alter_flags.any())
+        ha_alter_flags.is_set())
     {
       Alter_info tmp_alter_info(*alter_info, session->mem_root);
 

=== modified file 'drizzled/sql_update.cc'
--- drizzled/sql_update.cc	2009-05-08 19:13:49 +0000
+++ drizzled/sql_update.cc	2009-05-08 19:27:21 +0000
@@ -25,7 +25,6 @@
 #include <drizzled/sql_base.h>
 #include <drizzled/field/timestamp.h>
 
-#include <bitset>
 #include <list>
 
 using namespace std;
@@ -66,6 +65,7 @@
   return false;
 }
 
+
 /**
   Re-read record if more columns are needed for error message.
 
@@ -83,7 +83,8 @@
   Field **field_p;
   Field *field;
   uint32_t keynr;
-  bitset<MAX_FIELDS> unique_map; /* Fields in offended unique */
+  MY_BITMAP unique_map; /* Fields in offended unique. */
+  my_bitmap_map unique_map_buf[bitmap_buffer_size(MAX_FIELDS)];
 
   /*
     Only duplicate key errors print the key value.
@@ -101,33 +102,32 @@
     return;
 
   /* Create unique_map with all fields used by that index. */
+  bitmap_init(&unique_map, unique_map_buf, table->s->fields, false);
   table->mark_columns_used_by_index_no_reset(keynr, &unique_map);
 
   /* Subtract read_set and write_set. */
-  unique_map &= table->read_set->flip();
-  unique_map &= table->write_set->flip();
-  table->read_set->flip();
-  table->write_set->flip();
+  bitmap_subtract(&unique_map, table->read_set);
+  bitmap_subtract(&unique_map, table->write_set);
 
   /*
     If the unique index uses columns that are neither in read_set
     nor in write_set, we must re-read the record.
     Otherwise no need to do anything.
   */
-  if (unique_map.none())
+  if (bitmap_is_clear_all(&unique_map))
     return;
 
   /* Get identifier of last read record into table->file->ref. */
   table->file->position(table->record[0]);
   /* Add all fields used by unique index to read_set. */
-  *(table->read_set) |= unique_map;
+  bitmap_union(table->read_set, &unique_map);
   /* Tell the engine about the new set. */
   table->file->column_bitmaps_signal();
   /* Read record that is identified by table->file->ref. */
   (void) table->file->rnd_pos(table->record[1], table->file->ref);
   /* Copy the newly read columns into the new record. */
   for (field_p= table->field; (field= *field_p); field_p++)
-    if (unique_map.test(field->field_index))
+    if (bitmap_is_set(&unique_map, field->field_index))
       field->copy_from_tmp(table->s->rec_buff_length);
 
   return;
@@ -213,13 +213,15 @@
   if (table->timestamp_field)
   {
     // Don't set timestamp column if this is modified
-    if (table->write_set->test(table->timestamp_field->field_index))
+    if (bitmap_is_set(table->write_set,
+                      table->timestamp_field->field_index))
       table->timestamp_field_type= TIMESTAMP_NO_AUTO_SET;
     else
     {
       if (table->timestamp_field_type == TIMESTAMP_AUTO_SET_ON_UPDATE ||
           table->timestamp_field_type == TIMESTAMP_AUTO_SET_ON_BOTH)
-        table->write_set->set(table->timestamp_field->field_index);
+        bitmap_set_bit(table->write_set,
+                       table->timestamp_field->field_index);
     }
   }
 
@@ -253,7 +255,7 @@
       table->timestamp_field &&
       (table->timestamp_field_type == TIMESTAMP_AUTO_SET_ON_UPDATE ||
        table->timestamp_field_type == TIMESTAMP_AUTO_SET_ON_BOTH))
-    *(table->read_set) |= *(table->write_set);
+    bitmap_union(table->read_set, table->write_set);
   // Don't count on usage of 'only index' when calculating which key to use
   table->covering_keys.clear_all();
 
@@ -484,7 +486,7 @@
   */
   can_compare_record= (!(table->file->ha_table_flags() &
                          HA_PARTIAL_COLUMN_READ) ||
-                       isBitmapSubset(table->write_set, table->read_set));
+                       bitmap_is_subset(table->write_set, table->read_set));
 
   while (!(error=info.read_record(&info)) && !session->killed)
   {
@@ -850,7 +852,8 @@
     Table *table= tl->table;
     /* Only set timestamp column if this is not modified */
     if (table->timestamp_field &&
-        table->write_set->test(table->timestamp_field->field_index))
+        bitmap_is_set(table->write_set,
+                      table->timestamp_field->field_index))
       table->timestamp_field_type= TIMESTAMP_NO_AUTO_SET;
 
     /* if table will be updated then check that it is unique */
@@ -1322,8 +1325,8 @@
       bool can_compare_record;
       can_compare_record= (!(table->file->ha_table_flags() &
                              HA_PARTIAL_COLUMN_READ) ||
-                           isBitmapSubset(table->write_set,
-                                          table->write_set));
+                           bitmap_is_subset(table->write_set,
+                                            table->read_set));
       table->status|= STATUS_UPDATED;
       table->storeRecord();
       if (fill_record(session, *fields_for_table[offset],
@@ -1525,8 +1528,8 @@
 
     can_compare_record= (!(table->file->ha_table_flags() &
                            HA_PARTIAL_COLUMN_READ) ||
-                         isBitmapSubset(table->write_set,
-                                        table->read_set));
+                         bitmap_is_subset(table->write_set,
+                                          table->read_set));
 
     for (;;)
     {

=== modified file 'drizzled/table.cc'
--- drizzled/table.cc	2009-05-08 19:13:49 +0000
+++ drizzled/table.cc	2009-05-08 19:27:21 +0000
@@ -32,7 +32,6 @@
 #include <drizzled/field/varstring.h>
 #include <drizzled/field/double.h>
 #include <string>
-#include <bitset>
 
 #include <drizzled/unireg.h>
 #include <drizzled/message/table.pb.h>
@@ -1214,7 +1213,8 @@
   if (!(bitmaps= (my_bitmap_map*) alloc_root(&share->mem_root,
                                              share->column_bitmap_size)))
     goto err;
-  share->all_set.set();
+  bitmap_init(&share->all_set, bitmaps, share->fields, false);
+  bitmap_set_all(&share->all_set);
 
   if(handler_file)
     delete handler_file;
@@ -1490,6 +1490,16 @@
   }
 
   /* Allocate bitmaps */
+
+  bitmap_size= share->column_bitmap_size;
+  if (!(bitmaps= (unsigned char*) alloc_root(&outparam->mem_root, bitmap_size*3)))
+    goto err;
+  bitmap_init(&outparam->def_read_set,
+              (my_bitmap_map*) bitmaps, share->fields, false);
+  bitmap_init(&outparam->def_write_set,
+              (my_bitmap_map*) (bitmaps+bitmap_size), share->fields, false);
+  bitmap_init(&outparam->tmp_set,
+              (my_bitmap_map*) (bitmaps+bitmap_size*2), share->fields, false);
   outparam->default_column_bitmaps();
 
   /* The table struct is now initialized;  Open the table */
@@ -1989,12 +1999,17 @@
     a tmp_set bitmap to be used by things like filesort.
 */
 
-void Table::setup_tmp_table_column_bitmaps()
+void Table::setup_tmp_table_column_bitmaps(unsigned char *bitmaps)
 {
+  uint32_t field_count= s->fields;
+
+  bitmap_init(&this->def_read_set, (my_bitmap_map*) bitmaps, field_count, false);
+  bitmap_init(&this->tmp_set, (my_bitmap_map*) (bitmaps+ bitmap_buffer_size(field_count)), field_count, false);
+
   /* write_set and all_set are copies of read_set */
   def_write_set= def_read_set;
   s->all_set= def_read_set;
-  this->s->all_set.set();
+  bitmap_set_all(&this->s->all_set);
   default_column_bitmaps();
 }
 
@@ -2626,10 +2641,11 @@
 void Table::clear_column_bitmaps()
 {
   /*
-    Reset column read/write usage.
+    Reset column read/write usage. It's identical to:
+    bitmap_clear_all(&table->def_read_set);
+    bitmap_clear_all(&table->def_write_set);
   */
-  def_read_set.reset();
-  def_write_set.reset(); /* TODO: is this needed here? */
+  memset(def_read_set.bitmap, 0, s->column_bitmap_size*2);
   column_bitmaps_set(&def_read_set, &def_write_set);
 }
 
@@ -2669,10 +2685,10 @@
 
 void Table::mark_columns_used_by_index(uint32_t index)
 {
-  bitset<MAX_FIELDS> *bitmap= &tmp_set;
+  MY_BITMAP *bitmap= &tmp_set;
 
   (void) file->extra(HA_EXTRA_KEYREAD);
-  bitmap->reset();
+  bitmap_clear_all(bitmap);
   mark_columns_used_by_index_no_reset(index, bitmap);
   column_bitmaps_set(bitmap, bitmap);
   return;
@@ -2731,8 +2747,8 @@
     We must set bit in read set as update_auto_increment() is using the
     store() to check overflow of auto_increment values
   */
-  read_set->set(found_next_number_field->field_index);
-  write_set->set(found_next_number_field->field_index);
+  bitmap_set_bit(read_set, found_next_number_field->field_index);
+  bitmap_set_bit(write_set, found_next_number_field->field_index);
   if (s->next_number_keypart)
     mark_columns_used_by_index_no_reset(s->next_number_index, read_set);
   file->column_bitmaps_signal();
@@ -2765,7 +2781,7 @@
     for (reg_field= field ; *reg_field ; reg_field++)
     {
       if ((*reg_field)->flags & PART_KEY_FLAG)
-        read_set->set((*reg_field)->field_index);
+        bitmap_set_bit(read_set, (*reg_field)->field_index);
     }
     file->column_bitmaps_signal();
   }
@@ -2816,7 +2832,7 @@
     {
       /* Merge keys is all keys that had a column refered to in the query */
       if (merge_keys.is_overlapping((*reg_field)->part_of_key))
-        read_set->set((*reg_field)->field_index);
+        bitmap_set_bit(read_set, (*reg_field)->field_index);
     }
     file->column_bitmaps_signal();
   }
@@ -3233,7 +3249,7 @@
   uint32_t  copy_func_count= param->func_count;
   uint32_t  hidden_null_count, hidden_null_pack_length, hidden_field_count;
   uint32_t  blob_count,group_null_items, string_count;
-  uint32_t  temp_pool_slot= MY_BIT_NONE;
+  uint32_t  temp_pool_slot=MY_BIT_NONE;
   uint32_t fieldnr= 0;
   ulong reclength, string_total_length;
   bool  using_unique_constraint= 0;
@@ -3255,7 +3271,7 @@
   status_var_increment(session->status_var.created_tmp_tables);
 
   if (use_temp_pool && !(test_flags & TEST_KEEP_TMP_TABLES))
-    temp_pool_slot = temp_pool.setNextBit();
+    temp_pool_slot = bitmap_lock_set_next(&temp_pool);
 
   if (temp_pool_slot != MY_BIT_NONE) // we got a slot
     sprintf(path, "%s_%lx_%i", TMP_FILE_PREFIX,
@@ -3331,14 +3347,14 @@
                         NULL))
   {
     if (temp_pool_slot != MY_BIT_NONE)
-      temp_pool.resetBit(temp_pool_slot);
+      bitmap_lock_clear_bit(&temp_pool, temp_pool_slot);
     return(NULL);				/* purecov: inspected */
   }
   /* Copy_field belongs to Tmp_Table_Param, allocate it in Session mem_root */
   if (!(param->copy_field= copy= new (session->mem_root) Copy_field[field_count]))
   {
     if (temp_pool_slot != MY_BIT_NONE)
-      temp_pool.resetBit(temp_pool_slot);
+      bitmap_lock_clear_bit(&temp_pool, temp_pool_slot);
     free_root(&own_root, MYF(0));               /* purecov: inspected */
     return(NULL);				/* purecov: inspected */
   }
@@ -3360,7 +3376,7 @@
   table->reginfo.lock_type=TL_WRITE;	/* Will be updated */
   table->db_stat=HA_OPEN_KEYFILE+HA_OPEN_RNDFILE;
   table->map=1;
-  table->temp_pool_slot= temp_pool_slot;
+  table->temp_pool_slot = temp_pool_slot;
   table->copy_blobs= 1;
   table->in_use= session;
   table->quick_keys.init();
@@ -3592,7 +3608,7 @@
   copy_func[0]=0;				// End marker
   param->func_count= copy_func - param->items_to_copy;
 
-  table->setup_tmp_table_column_bitmaps();
+  table->setup_tmp_table_column_bitmaps(bitmaps);
 
   recinfo=param->start_recinfo;
   null_flags=(unsigned char*) table->record[0];
@@ -3890,7 +3906,7 @@
   session->mem_root= mem_root_save;
   table->free_tmp_table(session);                    /* purecov: inspected */
   if (temp_pool_slot != MY_BIT_NONE)
-    temp_pool.resetBit(temp_pool_slot);
+    bitmap_lock_clear_bit(&temp_pool, temp_pool_slot);
   return(NULL);				/* purecov: inspected */
 }
 
@@ -3945,7 +3961,7 @@
   share->blob_field= blob_field;
   share->fields= field_count;
   share->blob_ptr_size= portable_sizeof_char_ptr;
-  table->setup_tmp_table_column_bitmaps();
+  table->setup_tmp_table_column_bitmaps(bitmaps);
 
   /* Create all fields and calculate the total length of record */
   List_iterator_fast<Create_field> it(field_list);
@@ -4200,7 +4216,7 @@
   free_io_cache(this);
 
   if (temp_pool_slot != MY_BIT_NONE)
-    temp_pool.resetBit(temp_pool_slot);
+    bitmap_lock_clear_bit(&temp_pool, temp_pool_slot);
 
   free_root(&own_root, MYF(0)); /* the table is allocated in its own root */
   session->set_proc_info(save_proc_info);
@@ -4315,16 +4331,16 @@
   return(1);
 }
 
-bitset<MAX_FIELDS> *Table::use_all_columns(bitset<MAX_FIELDS> *bitmap)
+my_bitmap_map *Table::use_all_columns(MY_BITMAP *bitmap)
 {
-  bitset<MAX_FIELDS> *old= bitmap;
-  bitmap= &s->all_set;
+  my_bitmap_map *old= bitmap->bitmap;
+  bitmap->bitmap= s->all_set.bitmap;
   return old;
 }
 
-void Table::restore_column_map(bitset<MAX_FIELDS> *old)
+void Table::restore_column_map(my_bitmap_map *old)
 {
-  read_set= old;
+  read_set->bitmap= old;
 }
 
 uint32_t Table::find_shortest_key(const key_map *usable_keys)
@@ -4380,7 +4396,7 @@
   /* Compare updated fields */
   for (Field **ptr= field ; *ptr ; ptr++)
   {
-    if (write_set->test((*ptr)->field_index) &&
+    if (bitmap_is_set(write_set, (*ptr)->field_index) &&
 	(*ptr)->cmp_binary_offset(s->rec_buff_length))
       return true;
   }

=== modified file 'drizzled/table.h'
--- drizzled/table.h	2009-05-08 19:07:39 +0000
+++ drizzled/table.h	2009-05-08 19:27:21 +0000
@@ -33,7 +33,6 @@
 #include <drizzled/lex_string.h>
 #include <drizzled/table_list.h>
 #include <drizzled/table_share.h>
-#include <bitset>
 
 class Item;
 class Item_subselect;
@@ -95,7 +94,7 @@
 
   /* For TMP tables, should be pulled out as a class */
   void updateCreateInfo(HA_CREATE_INFO *create_info);
-  void setup_tmp_table_column_bitmaps();
+  void setup_tmp_table_column_bitmaps(unsigned char *bitmaps);
   bool create_myisam_tmp_table(KEY *keyinfo,
                                MI_COLUMNDEF *start_recinfo,
                                MI_COLUMNDEF **recinfo,
@@ -163,8 +162,8 @@
   const char	*alias;            	  /* alias or table name */
   unsigned char		*null_flags;
   my_bitmap_map	*bitmap_init_value;
-  std::bitset<MAX_FIELDS> def_read_set, def_write_set, tmp_set; /* containers */
-  std::bitset<MAX_FIELDS> *read_set, *write_set;                /* Active column sets */
+  MY_BITMAP     def_read_set, def_write_set, tmp_set; /* containers */
+  MY_BITMAP     *read_set, *write_set;          /* Active column sets */
   /*
    The ID of the query that opened and is using this table. Has different
    meanings depending on the table type.
@@ -291,31 +290,31 @@
   void reset_item_list(List<Item> *item_list) const;
   void clear_column_bitmaps(void);
   void prepare_for_position(void);
-  void mark_columns_used_by_index_no_reset(uint32_t index, std::bitset<MAX_FIELDS> *map);
+  void mark_columns_used_by_index_no_reset(uint32_t index, MY_BITMAP *map);
   void mark_columns_used_by_index(uint32_t index);
   void restore_column_maps_after_mark_index();
   void mark_auto_increment_column(void);
   void mark_columns_needed_for_update(void);
   void mark_columns_needed_for_delete(void);
   void mark_columns_needed_for_insert(void);
-  inline void column_bitmaps_set(std::bitset<MAX_FIELDS> *read_set_arg,
-                                 std::bitset<MAX_FIELDS> *write_set_arg)
+  inline void column_bitmaps_set(MY_BITMAP *read_set_arg,
+                                 MY_BITMAP *write_set_arg)
   {
     read_set= read_set_arg;
     write_set= write_set_arg;
     if (file)
       file->column_bitmaps_signal();
   }
-  inline void column_bitmaps_set_no_signal(std::bitset<MAX_FIELDS> *read_set_arg,
-                                           std::bitset<MAX_FIELDS> *write_set_arg)
+  inline void column_bitmaps_set_no_signal(MY_BITMAP *read_set_arg,
+                                           MY_BITMAP *write_set_arg)
   {
     read_set= read_set_arg;
     write_set= write_set_arg;
   }
 
-  void restore_column_map(std::bitset<MAX_FIELDS> *old);
+  void restore_column_map(my_bitmap_map *old);
 
-  std::bitset<MAX_FIELDS> *use_all_columns(std::bitset<MAX_FIELDS> *bitmap);
+  my_bitmap_map *use_all_columns(MY_BITMAP *bitmap);
   inline void use_all_columns()
   {
     column_bitmaps_set(&s->all_set, &s->all_set);

=== modified file 'drizzled/table_share.h'
--- drizzled/table_share.h	2009-05-08 19:07:39 +0000
+++ drizzled/table_share.h	2009-05-08 19:27:21 +0000
@@ -18,8 +18,6 @@
  *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
-#include <bitset>
-
 /*
   This class is shared between different table objects. There is one
   instance of table share per one table in the database.
@@ -57,7 +55,7 @@
   LEX_STRING comment;			/* Comment about table */
   const CHARSET_INFO *table_charset; /* Default charset of string fields */
 
-  std::bitset<MAX_FIELDS> all_set;
+  MY_BITMAP all_set;
   /*
     Key which is used for looking-up table in table cache and in the list
     of thread's temporary tables. Has the form of:

=== modified file 'plugin/csv/ha_tina.cc'
--- plugin/csv/ha_tina.cc	2009-04-29 07:16:04 +0000
+++ plugin/csv/ha_tina.cc	2009-05-08 19:27:21 +0000
@@ -602,7 +602,7 @@
     return(HA_ERR_END_OF_FILE);
 
   /* We must read all columns in case a table is opened for update */
-  read_all= !table->write_set->none();
+  read_all= !bitmap_is_clear_all(table->write_set);
   error= HA_ERR_CRASHED_ON_USAGE;
 
   memset(buf, 0, table->s->null_bytes);
@@ -672,7 +672,7 @@
       }
     }
 
-    if (read_all || table->read_set->test((*field)->field_index))
+    if (read_all || bitmap_is_set(table->read_set, (*field)->field_index))
     {
       if ((*field)->store(buffer.ptr(), buffer.length(), buffer.charset(),
                           CHECK_FIELD_WARN))

=== modified file 'plugin/innobase/handler/ha_innodb.cc'
--- plugin/innobase/handler/ha_innodb.cc	2009-05-08 19:07:39 +0000
+++ plugin/innobase/handler/ha_innodb.cc	2009-05-08 19:27:21 +0000
@@ -3688,8 +3688,8 @@
 				goto include_field;
 			}
 
-                        if (table->read_set->test(sql_idx) ||
-                            table->write_set->test(sql_idx)) {
+                        if (bitmap_is_set(table->read_set, sql_idx) ||
+                            bitmap_is_set(table->write_set, sql_idx)) {
 				/* This field is needed in the query */
 
 				goto include_field;

